/*
 *	termcap.c	1.1	20/7/87		agc	Joypace Ltd
 *
 *	Copyright Joypace Ltd, London, UK, 1987. All rights reserved.
 *	This file may be freely distributed provided that this notice
 *	remains attached.
 *
 *	A public domain implementation of the termcap(3) routines.
 */
#include <stdio.h>
#include <string.h>

#define CAPABLEN	2

#define ISSPACE(c)	((c) == ' ' || (c) == '\t' || (c) == '\r' || (c) == '\n')
#define ISDIGIT(x)	((x) >= '0' && (x) <= '9')

short	ospeed;			/* output speed */
char	PC;			/* padding character */
char	*BC;			/* back cursor movement */
char	*UP;			/* up cursor movement */

static char	*capab = 0;	/* the capability itself */
extern char	*getenv();	/* new, improved getenv */

/* copy entry in termcap file to buf */
static int get_entry(char *buf, FILE *file)
{
	int	c, last = 0;
	
	while ((c = fgetc(file)) != EOF)
		if ((*buf++ = c) == '\n')
			if (last == '\\')
				buf -= 2;
			else {
				*buf = 0;
				return 1;
			}
		else
			last = c;
	return 0;
}

/*
 *	tgetent - get the termcap entry for terminal name, and put it
 *	in bp (which must be an array of 1024 chars). Returns 1 if
 *	termcap entry found, 0 if not found, and -1 if file not found.
 */
int tgetent(char *bp, char *name)
{
	FILE	*fp;
	char	*file;
	char	*cp;
	int	cnt;

	capab = bp;
	if ((file = getenv("TERMCAP")) != 0) {
		if (*file != '/' &&
		    (cp = getenv("TERM")) != 0 && strcmp(name, cp) == 0) {
			(void) strcpy(bp, file);
			return 1;
		}
	} else
		file = "/etc/termcap";
	if ((fp = fopen(file, "r")) == 0) {
		capab = 0;			/* no valid termcap */
		return -1; 
	}

	while (get_entry(bp, fp)) {
	 	cp = bp;
		do {
			for (cnt = 0; name[cnt] != 0; cnt++, cp++)
				if (name[cnt] != *cp)
					break;
			if (name[cnt] != 0 || (*cp != '|' && *cp != ':'))
				goto next_name;
			fclose(fp);
			return 1;
next_name: 		while (*cp && *cp != '|' && *cp != ':')
				cp++;
		} while (cp++ == '|');
	}	
	capab = 0;				/* no valid termcap */
	fclose(fp);
	return 0;
}

/*
 *	tgetnum - get the numeric terminal capability corresponding
 *	to id. Returns the value, -1 if invalid.
 */
int tgetnum(char *id)
{
	char	*cp;

	if ((cp = capab) == 0 || id == 0)
		return -1;
	while (*cp++ != ':')			/* skip terminal name */
		;
	for (; *cp ; cp++) {
		if (*id == *cp++ && id[1] == *cp) {
			if (cp[1] != '#')
				return -1;
			return atoi(cp + 2);
		}
		while (*cp && *cp != ':')
			cp++;
	}
	return -1;
}

/*
 *	tgetflag - get the boolean flag corresponding to id. Returns -1
 *	if invalid, 0 if the flag is not in termcap entry, or 1 if it is
 *	present.
 */
int tgetflag(char *id)
{
	char	*cp;

	if ((cp = capab) == 0 || id == 0)
		return -1;
	while (*cp++ != ':')			/* skip terminal name */
		;
	for ( ; *cp ; cp++) {
		if (*id == *cp++ && id[1] == *cp)
			return 1;
		while (*cp && *cp != ':')
			cp++;
	}
	return 0;
}

/*
 *	tgetstr - get the string capability corresponding to id and place
 *	it in area (advancing area at same time). Expand escape sequences
 *	etc. Returns the string, or NULL if it can't do it.
 */
char *tgetstr(char *id, char **area)
{
	char	*cp, *wsp;			/* workspace == *area */
	char	*ret;
	int	i;

	if ((cp = capab) == 0 || id == 0)
		return 0;
	while (*cp++ != ':')			/* skip terminal name */
		;
	for ( ; *cp ; cp++) {
		if (*id == *cp++ && id[1] == *cp) {
			if (cp[1] != '=')
				return 0;
			cp += 2;
			for (ret = wsp = *area; *cp && *cp != ':' ; wsp++, cp++)
				switch (*cp) {
				case '^' :
					*wsp = *++cp - 'A';
					break;
				case '\\' :
					switch (*++cp) {
					case 'E' :
						*wsp = '\033';
						break;
					case 'n' :
						*wsp = '\n';
						break;
					case 'r' :
						*wsp = '\r';
						break;
					case 't' :
						*wsp = '\t';
						break;
					case 'b' :
						*wsp = '\b';
						break;
					case 'f' :
						*wsp = '\f';
						break;
					case '0' :
					case '1' :
					case '2' :
					case '3' :
						for (i = 0 ; *cp && ISDIGIT(*cp) ; cp++)
							i = i * 8 + *cp - '0';
						*wsp = i;
						cp--;
						break;
					case '^' :
					case '\\' :
						*wsp = *cp;
						break;
					}
					break;
				default :
					*wsp = *cp;
				}
			*area = wsp;
			*(*area)++ = '\0';
			return ret;
		}
		while (*cp && *cp != ':')
			cp++;
	}
	return 0;
}

/*
 *	tgoto - given the cursor motion string cm, make up the string
 *	for the cursor to go to (destcol, destline), and return the string.
 *	Returns "OOPS" if something's gone wrong, or the string otherwise.
 */
char *tgoto(char *cm, int destcol, int destline)
{
	char		*rp;
	int		*dp = &destline;
	int 		argno = 0, numval;
	static char	ret[24];
	static int	incr;		/* set by %i flag */

	for (rp = ret ; *cm ; cm++) {
		switch (*cm) {
		case '%' :
			switch (*++cm) {
			case '+' :
				if (dp == 0)
					return "OOPS";
				*rp++ = *dp + *++cm;
				dp = (dp == &destline) ? &destcol : 0;
				break;

			case '%' :
				*rp++ = '%';
				break;

			case 'i' :
				incr = 1;
				break;

			case 'd' :
				numval = (argno == 0 ? destline : destcol);
				numval += incr;
				argno++;
				*rp++ = '0' + (numval / 10);
				*rp++ = '0' + (numval % 10);
				break;
			}

			break;
		default :
			*rp++ = *cm;
		}
	}
	*rp = '\0';
	return ret;
}

/*
 *	tputs - put the string cp out onto the terminal, using the function
 *	outc. This should do padding for the terminal, but I can't find a
 *	terminal that needs padding at the moment...
 */
int
tputs(char *cp, int affcnt, int (*outc)())
{
	if (cp == 0)
		return 1;
	/* do any padding interpretation - left null for now */
	while (*cp)
		(*outc)(*cp++);
	return 1;
}
